"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EasyFormDialog = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const event_listener_1 = __importDefault(require("@use-it/event-listener"));
const iti_react_core_1 = require("@interface-technologies/iti-react-core");
const moment_timezone_1 = __importDefault(require("moment-timezone"));
const Dialog_1 = require("./Dialog");
/* eslint-disable */
/** @deprecated */
function formToObject(form) {
    const array = form.serializeArray();
    const obj = {};
    for (const pair of array) {
        obj[pair.name] = pair.value;
    }
    // serializeArray() ignores checkbox if it's unchecked and puts its value as "on"
    // if it is checked. This doesn't play well with web API so here we turn the
    // checkboxes into booleans.
    const checkboxes = form.find('[type="checkbox"]').toArray();
    for (const checkboxEl of checkboxes) {
        const checkbox = $(checkboxEl);
        const name = checkbox.attr('name');
        if (name) {
            obj[name] = checkbox.is(':checked');
        }
    }
    return obj;
}
/**
 * A wrapper around [[`ActionDialog`]] that removes a lot of the boilerplate needed
 * for dialogs that contain a form.
 *
 * ```tsx
 * interface ExampleProps {
 *     onSuccess(responseData: number): Promise<void>
 *     onClose(): void
 * }
 *
 * export function Example({
 *     onSuccess,
 *     onClose,
 * }: ExampleProps): ReactElement {
 *     const { onChildValidChange, allFieldsValid } = useFieldValidity()
 *     const [showValidation, setShowValidation] = useState(false)
 *     const vProps = { showValidation, onValidChange: onChildValidChange }
 *
 *     const [myNumber, setMyNumber] = useState('')
 *
 *     async function submit() {
 *         await api.product.performOperation()
 *
 *         return {
 *             responseData: parseInt(myNumber),
 *         }
 *     }
 *
 *     return (
 *         <EasyFormDialog
 *             title="Enter a Number"
 *             submitButtonText="Submit"
 *             formIsValid={allFieldsValid}
 *             showValidation={showValidation}
 *             onShowValidationChange={setShowValidation}
 *             onSubmit={submit}
 *             onSuccess={onSuccess}
 *             onClose={onClose}
 *         >
 *             <FormGroup label="My number">
 *                 {(id) => (
 *                     <ValidatedInput
 *                         id={id}
 *                         name="myNumber"
 *                         validators={[Validators.required(), Validators.integer()]}
 *                         value={myNumber}
 *                         onChange={setMyNumber}
 *                         {...vProps}
 *                     />
 *                 )}
 *             </FormGroup>
 *         </EasyFormDialog>
 *     )
 * }
 * ```
 */
function EasyFormDialog({ title, submitButtonText, submitEnabled: propsSubmitEnabled = true, submitButtonClass, cancelButtonText, formIsValid, showValidation, onShowValidationChange, onSuccess, modalClass, focusFirst, onClose, onSubmit, onCancel, showFooter, children, closeRef: propsCloseRef, }) {
    const internalCloseRef = (0, react_1.useRef)(() => { });
    const closeRef = propsCloseRef !== null && propsCloseRef !== void 0 ? propsCloseRef : internalCloseRef;
    const submitEnabled = propsSubmitEnabled && (0, iti_react_core_1.getSubmitEnabled)(formIsValid, showValidation);
    const { onError } = (0, react_1.useContext)(iti_react_core_1.ItiReactCoreContext);
    const [submitting, setSubmitting] = (0, react_1.useState)(false);
    const submittedTimeRef = (0, react_1.useRef)();
    const formRef = (0, react_1.useRef)(null);
    async function submit() {
        var _a;
        if (!submitEnabled)
            return;
        if (submitting)
            return;
        onShowValidationChange(true);
        if (!formIsValid)
            return;
        // Prevent double submit when Ctrl+Enter is pressed in Firefox
        if (submittedTimeRef.current &&
            (0, moment_timezone_1.default)().diff(submittedTimeRef.current, 'ms') < 200) {
            return;
        }
        submittedTimeRef.current = (0, moment_timezone_1.default)();
        setSubmitting(true);
        let formData = {};
        // jQuery is an optional peer dependency.
        // We have to be careful not to depend on @types/jquery either.
        /* eslint-disable */
        const jq = window.jQuery;
        if (jq) {
            if (!formRef.current)
                throw new Error('formRef.current is null.');
            formData = formToObject(jq(formRef.current));
        }
        /* eslint-enable */
        try {
            // hack to allow onSubmit to return void
            const onSubmitReturnValue = (await onSubmit(formData));
            const shouldClose = (_a = onSubmitReturnValue === null || onSubmitReturnValue === void 0 ? void 0 : onSubmitReturnValue.shouldClose) !== null && _a !== void 0 ? _a : true;
            const responseData = onSubmitReturnValue === null || onSubmitReturnValue === void 0 ? void 0 : onSubmitReturnValue.responseData;
            if (shouldClose) {
                // onSuccess may be loading data, so wait for it to finish before hiding the modal
                // and setting submitting=false
                await onSuccess(responseData);
                closeRef.current();
            }
        }
        catch (e) {
            onError(e);
            return;
        }
        setSubmitting(false);
    }
    // Submit form on Ctrl+Enter - convenient when you are typing in a textarea
    (0, event_listener_1.default)('keypress', (e) => {
        if (e.ctrlKey && e.code === 'Enter') {
            void submit();
        }
    });
    return ((0, jsx_runtime_1.jsx)(Dialog_1.ActionDialog, { closeRef: closeRef, title: title, actionButtonText: submitButtonText, actionButtonEnabled: submitEnabled, actionButtonClass: submitButtonClass, cancelButtonText: cancelButtonText, action: submit, actionInProgress: submitting, modalClass: modalClass, onClose: onClose, focusFirst: focusFirst, showFooter: showFooter, onCancel: onCancel, children: (0, jsx_runtime_1.jsxs)("form", { ref: formRef, onSubmit: async (e) => {
                e.preventDefault();
                await submit();
            }, noValidate: true, children: [children, (0, jsx_runtime_1.jsx)("input", { type: "submit", hidden: true }, void 0)] }, void 0) }, void 0));
}
exports.EasyFormDialog = EasyFormDialog;
