"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.usePaginationHelpers = void 0;
const react_1 = require("react");
const lodash_1 = require("lodash");
const iti_react_core_1 = require("@interface-technologies/iti-react-core");
/**
 * A hook that combines three things that need to be implemented when using pagination:
 *
 * - Resets page when filters change
 * - Returns total number of pages
 * - If the current page has no items and is not the first page, decrements the page
 *   (prevent non-existent pages). This situation occurs when someone deletes items
 *   while the user is viewing a paginated list.
 *
 * ```
 * const totalPages = usePaginationHelpers(...)
 * ```
 */
function usePaginationHelpers(options) {
    const { queryParams, onPageChange, firstPage, items, totalCount, pageSizeWhenItemsRetrieved, } = (0, lodash_1.defaults)(options, {
        firstPage: 1,
    });
    const prevQueryParamsRef = (0, react_1.useRef)();
    (0, react_1.useEffect)(() => {
        if (prevQueryParamsRef.current) {
            const newPage = (0, iti_react_core_1.resetPageIfFiltersChanged)(prevQueryParamsRef.current, queryParams, firstPage).page;
            if (queryParams.page !== newPage)
                onPageChange(newPage);
        }
        prevQueryParamsRef.current = queryParams;
    }, [queryParams, firstPage, onPageChange]);
    const pageHasItems = items.length !== 0;
    (0, react_1.useEffect)(() => {
        (0, iti_react_core_1.preventNonExistentPage)({
            page: queryParams.page,
            pageHasItems,
            onPageChange,
            firstPage,
        });
    }, [queryParams.page, pageHasItems, onPageChange, firstPage]);
    return (0, iti_react_core_1.getTotalPages)(totalCount, pageSizeWhenItemsRetrieved);
}
exports.usePaginationHelpers = usePaginationHelpers;
