"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TimeValidators = exports.TimeInput = exports.DefaultClearButtonComponent = exports.timeInputValueToDecimalHours = exports.timeInputValueFromDecimalHours = exports.defaultTimeInputValue = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = __importDefault(require("react"));
const moment_timezone_1 = __importDefault(require("moment-timezone"));
const iti_react_core_1 = require("@interface-technologies/iti-react-core");
const lodash_1 = require("lodash");
const validation_1 = require("../validation");
const select_1 = require("./select");
const LinkButton_1 = require("../components/LinkButton");
exports.defaultTimeInputValue = {
    hours: undefined,
    minutes: undefined,
    ampm: undefined,
};
/**
 * Example:
 *
 * ```
 * timeInputValueFromDecimalHours(9.75)
 * // returns { hours: 9, minutes: 45, ampm: 'am' }
 * ```
 *
 * @returns a [[`TimeInputValue`]] or `undefined` if `decimalHours` is `undefined`.
 */
function timeInputValueFromDecimalHours(decimalHours) {
    if (typeof decimalHours === 'undefined')
        return exports.defaultTimeInputValue;
    const { hours, minutes } = (0, iti_react_core_1.toHoursAndMinutes)(decimalHours);
    const d = (0, moment_timezone_1.default)().hour(hours).minute(minutes);
    return {
        hours: parseInt(d.format('h')),
        minutes: d.minute(),
        ampm: d.format('a'),
    };
}
exports.timeInputValueFromDecimalHours = timeInputValueFromDecimalHours;
function timeInputValueToDecimalHours(value) {
    if (typeof value.hours === 'undefined' ||
        typeof value.minutes === 'undefined' ||
        typeof value.ampm === 'undefined') {
        return undefined;
    }
    let hours = value.hours % 12;
    if (value.ampm === 'pm')
        hours += 12;
    return (0, iti_react_core_1.toDecimalHours)(hours, value.minutes);
}
exports.timeInputValueToDecimalHours = timeInputValueToDecimalHours;
//
// Supporting functions for component
//
const basicValidator = (value) => {
    const { hours, minutes, ampm } = value;
    const types = [typeof hours, typeof minutes, typeof ampm];
    let undefinedCount = 0;
    for (const type of types) {
        if (type === 'undefined')
            undefinedCount += 1;
    }
    if (undefinedCount !== 0 && undefinedCount !== 3)
        return 'You must enter a valid time or leave all fields blank.';
    return undefined;
};
const toOption = (x) => ({ value: x, label: x.toString() });
const options = {
    hours: [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12].map(toOption),
    minutes: ['00', '15', '30', '45'].map((m) => ({ value: parseInt(m), label: m })),
    ampm: ['am', 'pm'].map(toOption),
};
/**
 * The default clear button used in [[`TimeInput`]]. You can provide a different
 * clear button component if you want.
 */
function DefaultClearButtonComponent({ onClick, enabled, }) {
    if (!enabled) {
        return (0, jsx_runtime_1.jsx)("span", { className: "default-clear-button disabled", children: "Clear" }, void 0);
    }
    return ((0, jsx_runtime_1.jsx)(LinkButton_1.LinkButton, { onClick: onClick, className: "default-clear-button", children: "Clear" }, void 0));
}
exports.DefaultClearButtonComponent = DefaultClearButtonComponent;
function fromSelectValue(selectValue) {
    if (selectValue === '' || selectValue === null)
        return undefined;
    return selectValue;
}
/**
 * A time of day input with dropdowns for hours, minutes, and am/pm.
 *
 * Make sure to pass `individualInputsRequired=true` and `isClearable=false`
 * as well as `TimeValidators.required()` if the time is required.
 *
 * It's not the most performant or user-friendly component but it has gotten the job
 * done so far.
 */
exports.TimeInput = react_1.default.memo(({ showValidation, enabled = true, name, isClearable = true, clearButtonComponent: ClearButton = DefaultClearButtonComponent, individualInputsRequired, ...otherProps }) => {
    const { value, onChange } = (0, iti_react_core_1.useControlledValue)({
        value: otherProps.value,
        onChange: otherProps.onChange,
        defaultValue: otherProps.defaultValue,
        fallbackValue: exports.defaultTimeInputValue,
    });
    function onHoursChange(selectValue) {
        const hours = fromSelectValue(selectValue);
        if (typeof hours !== 'number')
            throw new Error('Hours is not a number.');
        onChange({
            ...value,
            hours,
        });
    }
    function onMinutesChange(selectValue) {
        const minutes = fromSelectValue(selectValue);
        if (typeof minutes !== 'number')
            throw new Error('Hours is not a number.');
        onChange({
            ...value,
            minutes,
        });
    }
    function onAmpmChange(selectValue) {
        const ampm = fromSelectValue(selectValue);
        if (!(ampm === 'am' || ampm === 'pm' || typeof ampm === 'undefined'))
            throw new Error('ampm is not am, pm , or undefined.');
        onChange({
            ...value,
            ampm,
        });
    }
    const validatorOutput = (0, iti_react_core_1.useValidation)({
        value,
        name,
        onValidChange: otherProps.onValidChange,
        validators: [basicValidator, ...otherProps.validators],
        validationKey: otherProps.validationKey,
        asyncValidator: otherProps.asyncValidator,
        onAsyncError: otherProps.onAsyncError,
        formLevelValidatorOutput: otherProps.formLevelValidatorOutput,
    });
    const { hours, minutes, ampm } = value;
    const indiviudalInputValidators = [];
    if (individualInputsRequired) {
        // don't display any feedback under individual fields
        indiviudalInputValidators.push((value) => value === null ? iti_react_core_1.INVALID_NO_FEEDBACK : undefined);
    }
    const commonProps = {
        showValidation,
        validators: indiviudalInputValidators,
        enabled,
    };
    return ((0, jsx_runtime_1.jsx)("div", { className: "time-input", children: (0, jsx_runtime_1.jsx)(validation_1.ValidationFeedback, { validatorOutput: validatorOutput, showValidation: showValidation, children: (0, jsx_runtime_1.jsxs)("div", { className: "flex-container", children: [(0, jsx_runtime_1.jsx)("div", { className: "input", children: (0, jsx_runtime_1.jsx)(select_1.ValidatedSelect, { ...commonProps, name: `${name}_hours`, value: (0, iti_react_core_1.undefinedToNull)(hours), onChange: onHoursChange, options: options.hours, placeholder: "HH", "aria-label": "Hours" }, void 0) }, void 0), (0, jsx_runtime_1.jsx)("div", { className: "input", children: (0, jsx_runtime_1.jsx)(select_1.ValidatedSelect, { ...commonProps, name: `${name}_minutes`, value: (0, iti_react_core_1.undefinedToNull)(minutes), onChange: onMinutesChange, options: options.minutes, placeholder: "mm", "aria-label": "Minutes" }, void 0) }, void 0), (0, jsx_runtime_1.jsx)("div", { className: "input", children: (0, jsx_runtime_1.jsx)(select_1.ValidatedSelect, { ...commonProps, name: `${name}_ampm`, value: (0, iti_react_core_1.undefinedToNull)(ampm), onChange: onAmpmChange, options: options.ampm, placeholder: "", "aria-label": "AM or PM" }, void 0) }, void 0), isClearable && !(0, lodash_1.isEqual)(value, exports.defaultTimeInputValue) && ((0, jsx_runtime_1.jsx)(ClearButton, { onClick: () => onChange(exports.defaultTimeInputValue), enabled: enabled }, void 0))] }, void 0) }, void 0) }, void 0));
});
function required() {
    return (value) => {
        const { hours, minutes, ampm } = value;
        if (typeof hours === 'undefined' ||
            typeof minutes === 'undefined' ||
            typeof ampm === 'undefined')
            return iti_react_core_1.Validators.required()('');
        return undefined;
    };
}
/** Validators for use with [[`TimeInput`]]. */
exports.TimeValidators = {
    required,
};
