"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorRouteSynchronizer = void 0;
const react_1 = require("react");
// eslint-disable-next-line no-restricted-imports
const react_router_dom_1 = require("react-router-dom");
const iti_react_core_1 = require("@interface-technologies/iti-react-core");
/**
 * A React component that adds a URL search param (usually `error`) if the `error`
 * prop is non-null. If the `error` prop is null, the URL search param is removed.
 *
 * Usually, the `error` prop will come from the Redux store. This allows you to display
 * an error page whenever an error occurs anywhere in your application.
 *
 * ### Test Cases
 *
 * 1. When an error occurs, the error param is added to the URL and the error page is
 *    displayed.
 * 2. When the URL contains the error param and user refreshes the page, the error
 *    param is removed and the error page is not displayed.
 * 3. When the user is on the error page and clicks back, they are returned to the page
 *    they were on when the error occurred.
 * 4. This tests the errorKey functionality.
 *    a. Error occurs, and error page is displayed.
 *    b. User navigates to some other page.
 *    c. Another error occurs. Make sure the error page is displayed.
 */
function ErrorRouteSynchronizer({ errorUrlParamName, error, }) {
    const navigate = (0, react_router_dom_1.useNavigate)();
    const location = (0, react_router_dom_1.useLocation)();
    const prevError = (0, iti_react_core_1.usePrevious)(error);
    (0, react_1.useEffect)(() => {
        const urlSearchParams = new URLSearchParams(location.search);
        const errorUrlParamExists = urlSearchParams.has(errorUrlParamName);
        if (error) {
            if (error !== prevError && !errorUrlParamExists) {
                // Add error URL param if an error was just set in the Redux state
                urlSearchParams.append(errorUrlParamName, '');
                // Push so that clicking back takes you to the page you were on
                navigate(`${location.pathname}?${urlSearchParams.toString()}`);
            }
        }
        else if (errorUrlParamExists) {
            // Remove error URL param if no error in Redux state
            urlSearchParams.delete(errorUrlParamName);
            navigate(`${location.pathname}?${urlSearchParams.toString()}`, {
                replace: true,
            });
        }
    }, [error, prevError, navigate, location, errorUrlParamName]);
    return null;
}
exports.ErrorRouteSynchronizer = ErrorRouteSynchronizer;
